@@ -1,4 +1,5 @@
 #include "cfl_table.h"
+#include "cfl_lock.hpp"
 
 #include <FL/Fl.H>
 #include <FL/Fl_Image.H>
@@ -94,58 +95,68 @@
         LOCK(self->table_box((Fl_Boxtype)val);)                                                    \
     }                                                                                              \
     int table##_table_box(table *self) {                                                           \
-        return self->table_box();                                                                  \
+        LOCK(auto ret = self->table_box(););                                                       \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_rows(table *self, int val) {                                                  \
         LOCK(self->rows(val);)                                                                     \
     }                                                                                              \
     int table##_rows(table *self) {                                                                \
-        return self->rows();                                                                       \
+        LOCK(auto ret = self->rows(););                                                            \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_cols(table *self, int val) {                                                  \
         LOCK(self->cols(val);)                                                                     \
     }                                                                                              \
     int table##_cols(table *self) {                                                                \
-        return self->cols();                                                                       \
+        LOCK(auto ret = self->cols(););                                                            \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_visible_cells(table *self, int *r1, int *r2, int *c1, int *c2) {                  \
         LOCK(self->visible_cells(*r1, *r2, *c1, *c2);)                                             \
     }                                                                                              \
     int table##_is_interactive_resize(table *self) {                                               \
-        return self->is_interactive_resize();                                                      \
+        LOCK(auto ret = self->is_interactive_resize(););                                           \
+        return ret;                                                                                \
     }                                                                                              \
     int table##_row_resize(table *self) {                                                          \
-        return self->row_resize();                                                                 \
+        LOCK(auto ret = self->row_resize(););                                                      \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_row_resize(table *self, int flag) {                                           \
         LOCK(self->row_resize(flag);)                                                              \
     }                                                                                              \
     int table##_col_resize(table *self) {                                                          \
-        return self->col_resize();                                                                 \
+        LOCK(auto ret = self->col_resize(););                                                      \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_col_resize(table *self, int flag) {                                           \
         LOCK(self->col_resize(flag);)                                                              \
     }                                                                                              \
     int table##_col_resize_min(table *self) {                                                      \
-        return self->col_resize_min();                                                             \
+        LOCK(auto ret = self->col_resize_min(););                                                  \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_col_resize_min(table *self, int val) {                                        \
         LOCK(self->col_resize_min(val);)                                                           \
     }                                                                                              \
     int table##_row_resize_min(table *self) {                                                      \
-        return self->row_resize_min();                                                             \
+        LOCK(auto ret = self->row_resize_min(););                                                  \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_row_resize_min(table *self, int val) {                                        \
         LOCK(self->row_resize_min(val);)                                                           \
     }                                                                                              \
     int table##_row_header(table *self) {                                                          \
-        return self->row_header();                                                                 \
+        LOCK(auto ret = self->row_header(););                                                      \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_row_header(table *self, int flag) {                                           \
         LOCK(self->row_header(flag);)                                                              \
     }                                                                                              \
     int table##_col_header(table *self) {                                                          \
-        return self->col_header();                                                                 \
+        LOCK(auto ret = self->col_header(););                                                      \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_col_header(table *self, int flag) {                                           \
         LOCK(self->col_header(flag);)                                                              \
@@ -154,37 +165,43 @@
         LOCK(self->col_header_height(height);)                                                     \
     }                                                                                              \
     int table##_col_header_height(table *self) {                                                   \
-        return self->col_header_height();                                                          \
+        LOCK(auto ret = self->col_header_height(););                                               \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_row_header_width(table *self, int width) {                                    \
         LOCK(self->row_header_width(width);)                                                       \
     }                                                                                              \
     int table##_row_header_width(table *self) {                                                    \
-        return self->row_header_width();                                                           \
+        LOCK(auto ret = self->row_header_width(););                                                \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_row_header_color(table *self, unsigned int val) {                             \
         LOCK(self->row_header_color(val);)                                                         \
     }                                                                                              \
     unsigned int table##_row_header_color(table *self) {                                           \
-        return self->row_header_color();                                                           \
+        LOCK(auto ret = self->row_header_color(););                                                \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_col_header_color(table *self, unsigned int val) {                             \
         LOCK(self->col_header_color(val);)                                                         \
     }                                                                                              \
     unsigned int table##_col_header_color(table *self) {                                           \
-        return self->col_header_color();                                                           \
+        LOCK(auto ret = self->col_header_color(););                                                \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_row_height(table *self, int row, int height) {                                \
         LOCK(self->row_height(row, height);)                                                       \
     }                                                                                              \
     int table##_row_height(table *self, int row) {                                                 \
-        return self->row_height(row);                                                              \
+        LOCK(auto ret = self->row_height(row););                                                   \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_col_width(table *self, int col, int width) {                                  \
         LOCK(self->col_width(col, width);)                                                         \
     }                                                                                              \
     int table##_col_width(table *self, int col) {                                                  \
-        return self->col_width(col);                                                               \
+        LOCK(auto ret = self->col_width(col););                                                    \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_row_height_all(table *self, int height) {                                     \
         LOCK(self->row_height_all(height);)                                                        \
@@ -199,19 +216,23 @@
         LOCK(self->col_position(col);)                                                             \
     }                                                                                              \
     int table##_row_position(table *self) {                                                        \
-        return self->row_position();                                                               \
+        LOCK(auto ret = self->row_position(););                                                    \
+        return ret;                                                                                \
     }                                                                                              \
     int table##_col_position(table *self) {                                                        \
-        return self->col_position();                                                               \
+        LOCK(auto ret = self->col_position(););                                                    \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_top_row(table *self, int row) {                                               \
         LOCK(self->top_row(row);)                                                                  \
     }                                                                                              \
     int table##_top_row(table *self) {                                                             \
-        return self->top_row();                                                                    \
+        LOCK(auto ret = self->top_row(););                                                         \
+        return ret;                                                                                \
     }                                                                                              \
     int table##_is_selected(table *self, int r, int c) {                                           \
-        return self->is_selected(r, c);                                                            \
+        LOCK(auto ret = self->is_selected(r, c););                                                 \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_get_selection(table *self, int *row_top, int *col_left, int *row_bot,             \
                                int *col_right) {                                                   \
@@ -222,13 +243,16 @@
         LOCK(self->set_selection(row_top, col_left, row_bot, col_right);)                          \
     }                                                                                              \
     int table##_move_cursor_with_shiftselect(table *self, int R, int C, int shiftselect) {         \
-        return self->move_cursor(R, C, shiftselect);                                               \
+        LOCK(auto ret = self->move_cursor(R, C, shiftselect););                                    \
+        return ret;                                                                                \
     }                                                                                              \
     int table##_move_cursor(table *self, int R, int C) {                                           \
-        return self->move_cursor(R, C);                                                            \
+        LOCK(auto ret = self->move_cursor(R, C););                                                 \
+        return ret;                                                                                \
     }                                                                                              \
     int table##_scrollbar_size(const table *self) {                                                \
-        return self->scrollbar_size();                                                             \
+        LOCK(auto ret = self->scrollbar_size(););                                                  \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_set_scrollbar_size(table *self, int newSize) {                                    \
         LOCK(self->scrollbar_size(newSize);)                                                       \
@@ -237,7 +261,8 @@
         LOCK(self->tab_cell_nav(val);)                                                             \
     }                                                                                              \
     int table##_tab_cell_nav(const table *self) {                                                  \
-        return self->tab_cell_nav();                                                               \
+        LOCK(auto ret = self->tab_cell_nav(););                                                    \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_draw_cell(table *self,                                                            \
                            void (*cb)(Fl_Widget *, int, int, int, int, int, int, int, void *),     \
@@ -246,19 +271,23 @@
              ((table##_Derived *)self)->set_cell_drawer(cb);)                                      \
     }                                                                                              \
     void *table##_draw_cell_data(const table *self) {                                              \
-        return ((table##_Derived *)self)->draw_cell_data_;                                         \
+        LOCK(auto ret = ((table##_Derived *)self)->draw_cell_data_);                               \
+        return ret;                                                                                \
     }                                                                                              \
     void table##_draw_cell_data(table *self, void *data) {                                         \
         LOCK(((table##_Derived *)self)->draw_cell_data_ = data)                                    \
     }                                                                                              \
     int table##_callback_col(table *self) {                                                        \
-        return self->callback_col();                                                               \
+        LOCK(auto ret = self->callback_col(););                                                    \
+        return ret;                                                                                \
     }                                                                                              \
     int table##_callback_row(table *self) {                                                        \
-        return self->callback_row();                                                               \
+        LOCK(auto ret = self->callback_row(););                                                    \
+        return ret;                                                                                \
     }                                                                                              \
     int table##_callback_context(table *self) {                                                    \
-        return self->callback_context();                                                           \
+        LOCK(auto ret = self->callback_context(););                                                \
+        return ret;                                                                                \
     }
 
 TABLE_CLASS(Fl_Table)
@@ -278,12 +307,11 @@ GROUP_DEFINE(Fl_Table_Row)
 TABLE_DEFINE(Fl_Table_Row)
 
 int Fl_Table_Row_row_selected(Fl_Table_Row *self, int row) {
-    return self->row_selected(row);
+    LOCK(auto ret = self->row_selected(row)); return ret;
 }
 
 int Fl_Table_Row_select_row(Fl_Table_Row *self, int row, int flag) {
-    int ret = 0;
-    LOCK(ret = self->select_row(row, flag);)
+    LOCK(auto ret = self->select_row(row, flag);)
     return ret;
 }
 
