@@ -2,6 +2,7 @@
 #define FL_INTERNALS
 
 #include "cfl_window.h"
+#include "cfl_lock.hpp"
 
 #include <FL/Fl.H>
 #include <FL/Fl_Double_Window.H>
@@ -31,30 +32,28 @@
         LOCK(self->icon((const Fl_RGB_Image *)((Fl_Image *)image));)                               \
     }                                                                                              \
     void *widget##_icon(const widget *self) {                                                      \
-        return (Fl_Image *)self->icon();                                                           \
+        LOCK(auto ret = (Fl_Image *)self->icon());                                                 \
+        return ret;                                                                                \
     }                                                                                              \
     void widget##_set_cursor(widget *self, int cursor) {                                           \
         LOCK(self->cursor((Fl_Cursor)cursor);)                                                     \
     }                                                                                              \
     int widget##_shown(widget *self) {                                                             \
-        return self->shown();                                                                      \
+        LOCK(auto ret = self->shown(););                                                           \
+        return ret;                                                                                \
     }                                                                                              \
     void *widget##_raw_handle(const widget *w) {                                                   \
-        Window temp = fl_xid(w);                                                                   \
-        if (!temp) {                                                                               \
-            return NULL;                                                                           \
-        }                                                                                          \
-        Window *xid = (Window *)malloc(sizeof(Window));                                            \
-        if (!xid)                                                                                  \
-            return NULL;                                                                           \
-        memcpy(xid, &temp, sizeof(Window));                                                        \
+        LOCK(Window temp = fl_xid(w);                                                              \
+             if (!temp) { return NULL; } Window *xid = (Window *)malloc(sizeof(Window));           \
+             if (!xid) return NULL; memcpy(xid, &temp, sizeof(Window));)                           \
         return xid;                                                                                \
     }                                                                                              \
     void widget##_set_border(widget *self, int flag) {                                             \
         LOCK(self->border(flag);)                                                                  \
     }                                                                                              \
     int widget##_border(const widget *self) {                                                      \
-        return self->border();                                                                     \
+        LOCK(auto ret = self->border(););                                                          \
+        return ret;                                                                                \
     }                                                                                              \
     void *widget##_region(const widget *self) {                                                    \
         Fl_X *t = Fl_X::i(self);                                                                   \
@@ -69,16 +68,19 @@
         LOCK(self->iconize())                                                                      \
     }                                                                                              \
     unsigned int widget##_fullscreen_active(const widget *self) {                                  \
-        return self->fullscreen_active();                                                          \
+        LOCK(auto ret = self->fullscreen_active(););                                               \
+        return ret;                                                                                \
     }                                                                                              \
     void widget##_free_position(widget *self) {                                                    \
         LOCK(self->free_position())                                                                \
     }                                                                                              \
     int widget##_decorated_w(const widget *self) {                                                 \
-        return self->decorated_w();                                                                \
+        LOCK(auto ret = self->decorated_w(););                                                     \
+        return ret;                                                                                \
     }                                                                                              \
     int widget##_decorated_h(const widget *self) {                                                 \
-        return self->decorated_h();                                                                \
+        LOCK(auto ret = self->decorated_h(););                                                     \
+        return ret;                                                                                \
     }                                                                                              \
     void widget##_size_range(widget *self, int minw, int minh, int maxw, int maxh) {               \
         LOCK(self->size_range(minw, minh, maxw, maxh))                                             \
@@ -91,16 +93,19 @@
              else self->shape(((Fl_Image *)image)->copy()); delete old;)                           \
     }                                                                                              \
     const void *widget##_shape(widget *self) {                                                     \
-        auto temp = self->shape();                                                                 \
+        LOCK(auto temp = self->shape());                                                           \
         if (!temp)                                                                                 \
             return NULL;                                                                           \
-        return ((Fl_Image *)temp)->copy();                                                         \
+        LOCK(auto ret = ((Fl_Image *)temp)->copy());                                               \
+        return ret;                                                                                \
     }                                                                                              \
     int widget##_x_root(const widget *self) {                                                      \
-        return self->x_root();                                                                     \
+        LOCK(auto ret = self->x_root(););                                                          \
+        return ret;                                                                                \
     }                                                                                              \
     int widget##_y_root(const widget *self) {                                                      \
-        return self->y_root();                                                                     \
+        LOCK(auto ret = self->y_root(););                                                          \
+        return ret;                                                                                \
     }                                                                                              \
     void widget##_set_cursor_image(widget *self, const void *image, int hot_x, int hot_y) {        \
         LOCK(self->cursor((const Fl_RGB_Image *)image, hot_x, hot_y);)                             \
@@ -109,7 +114,8 @@
         LOCK(self->default_cursor((Fl_Cursor)cursor);)                                             \
     }                                                                                              \
     int widget##_screen_num(widget *self) {                                                        \
-        return self->screen_num();                                                                 \
+        LOCK(auto ret = self->screen_num(););                                                      \
+        return ret;                                                                                \
     }                                                                                              \
     void widget##_set_screen_num(widget *self, int screen_num) {                                   \
         LOCK(self->screen_num(screen_num);)                                                        \
@@ -130,7 +136,8 @@ GROUP_DEFINE(Fl_Window)
 WINDOW_DEFINE(Fl_Window)
 
 Fl_Window *Fl_Window_find_by_handle(void *handle) {
-    return fl_find(*(Window *)handle);
+    LOCK(auto ret = fl_find(*(Window *)handle));
+    return ret;
 }
 
 winid resolve_raw_handle(void *handle) {
@@ -232,10 +239,7 @@ struct Fl_Overlay_Window_Derived : public Fl_Overlay_Window {
     virtual void resize(int x, int y, int w, int h) override {
         Fl_Overlay_Window::resize(x, y, w, h);
         if (this->as_window() == this->top_window()) {
-            Fl::lock();
-            Fl::handle(28, this->top_window());
-            Fl::unlock();
-            Fl::awake();
+            LOCK(Fl::handle(28, this->top_window()))
         }
     }
 
@@ -321,11 +325,12 @@ void Fl_Overlay_Window_draw_overlay(Fl_Overlay_Window *self, custom_draw_callbac
 }
 
 void Fl_Overlay_Window_redraw_overlay(Fl_Overlay_Window *self) {
-    self->redraw_overlay();
+    LOCK(self->redraw_overlay());
 }
 
 int Fl_Overlay_Window_can_do_overlay(Fl_Overlay_Window *self) {
-    return self->can_do_overlay();
+    LOCK(auto ret = self->can_do_overlay());
+    return ret;
 }
 
 WINDOW_DEFINE(Fl_Overlay_Window)
@@ -348,7 +353,8 @@ void Fl_Gl_Window_flush(Fl_Gl_Window *self) {
 }
 
 char Fl_Gl_Window_valid(const Fl_Gl_Window *self) {
-    return self->valid();
+    LOCK(auto ret = self->valid());
+    return ret;
 }
 
 void Fl_Gl_Window_set_valid(Fl_Gl_Window *self, char v) {
@@ -356,7 +362,8 @@ void Fl_Gl_Window_set_valid(Fl_Gl_Window *self, char v) {
 }
 
 char Fl_Gl_Window_context_valid(const Fl_Gl_Window *self) {
-    return self->context_valid();
+    LOCK(auto ret = self->context_valid());
+    return ret;
 }
 
 void Fl_Gl_Window_set_context_valid(Fl_Gl_Window *self, char v) {
@@ -364,11 +371,13 @@ void Fl_Gl_Window_set_context_valid(Fl_Gl_Window *self, char v) {
 }
 
 int Fl_Gl_Window_can_do(Fl_Gl_Window *self) {
-    return self->can_do();
+    LOCK(auto ret = self->can_do());
+    return ret;
 }
 
 void *Fl_Gl_Window_context(const Fl_Gl_Window *self) {
-    return self->context();
+    LOCK(auto ret = self->context());
+    return ret;
 }
 
 void Fl_Gl_Window_set_context(Fl_Gl_Window *self, void *ctx, int destroy_flag) {
@@ -384,7 +393,7 @@ void Fl_Gl_Window_ortho(Fl_Gl_Window *self) {
 }
 
 int Fl_Gl_Window_can_do_overlay(Fl_Gl_Window *self) {
-    return self->can_do_overlay();
+    LOCK(auto ret = self->can_do_overlay();) return ret;
 }
 
 void Fl_Gl_Window_redraw_overlay(Fl_Gl_Window *self) {
@@ -400,27 +409,31 @@ void Fl_Gl_Window_make_overlay_current(Fl_Gl_Window *self) {
 }
 
 float Fl_Gl_Window_pixels_per_unit(Fl_Gl_Window *self) {
-    return self->pixels_per_unit();
+    LOCK(auto ret = self->pixels_per_unit());
+    return ret;
 }
 
 int Fl_Gl_Window_pixel_w(Fl_Gl_Window *self) {
-    return self->pixel_w();
+    LOCK(auto ret = self->pixel_w());
+    return ret;
 }
 
 int Fl_Gl_Window_pixel_h(Fl_Gl_Window *self) {
-    return self->pixel_h();
+    LOCK(auto ret = self->pixel_h());
+    return ret;
 }
 
 int Fl_Gl_Window_mode(const Fl_Gl_Window *self) {
-    return self->mode();
+    LOCK(auto ret = self->mode());
+    return ret;
 }
 
 void Fl_Gl_Window_set_mode(Fl_Gl_Window *self, int mode) {
-    self->mode(mode);
+    LOCK(self->mode(mode));
 }
 
 void *Fl_Gl_Window_get_proc_address(Fl_Gl_Window *self, const char *s) {
-    return (void *)glutGetProcAddress(s);
+    LOCK(auto ret = (void *)glutGetProcAddress(s)); return ret;
 }
 
 WIDGET_CLASS(Fl_Glut_Window)
@@ -436,7 +449,8 @@ void Fl_Glut_Window_flush(Fl_Glut_Window *self) {
 }
 
 char Fl_Glut_Window_valid(const Fl_Glut_Window *self) {
-    return self->valid();
+    LOCK(auto ret = self->valid());
+    return ret;
 }
 
 void Fl_Glut_Window_set_valid(Fl_Glut_Window *self, char v) {
@@ -444,7 +458,8 @@ void Fl_Glut_Window_set_valid(Fl_Glut_Window *self, char v) {
 }
 
 char Fl_Glut_Window_context_valid(const Fl_Glut_Window *self) {
-    return self->context_valid();
+    LOCK(auto ret = self->context_valid());
+    return ret;
 }
 
 void Fl_Glut_Window_set_context_valid(Fl_Glut_Window *self, char v) {
@@ -452,11 +467,13 @@ void Fl_Glut_Window_set_context_valid(Fl_Glut_Window *self, char v) {
 }
 
 int Fl_Glut_Window_can_do(Fl_Glut_Window *self) {
-    return self->can_do();
+    LOCK(auto ret = self->can_do());
+    return ret;
 }
 
 void *Fl_Glut_Window_context(const Fl_Glut_Window *self) {
-    return self->context();
+    LOCK(auto ret = self->context());
+    return ret;
 }
 
 void Fl_Glut_Window_set_context(Fl_Glut_Window *self, void *ctx, int destroy_flag) {
@@ -472,7 +489,8 @@ void Fl_Glut_Window_ortho(Fl_Glut_Window *self) {
 }
 
 int Fl_Glut_Window_can_do_overlay(Fl_Glut_Window *self) {
-    return self->can_do_overlay();
+    LOCK(auto ret = self->can_do_overlay());
+    return ret;
 }
 
 void Fl_Glut_Window_redraw_overlay(Fl_Glut_Window *self) {
@@ -488,27 +506,31 @@ void Fl_Glut_Window_make_overlay_current(Fl_Glut_Window *self) {
 }
 
 float Fl_Glut_Window_pixels_per_unit(Fl_Glut_Window *self) {
-    return self->pixels_per_unit();
+    LOCK(auto ret = self->pixels_per_unit());
+    return ret;
 }
 
 int Fl_Glut_Window_pixel_w(Fl_Glut_Window *self) {
-    return self->pixel_w();
+    LOCK(auto ret = self->pixel_w());
+    return ret;
 }
 
 int Fl_Glut_Window_pixel_h(Fl_Glut_Window *self) {
-    return self->pixel_h();
+    LOCK(auto ret = self->pixel_h());
+    return ret;
 }
 
 int Fl_Glut_Window_mode(const Fl_Glut_Window *self) {
-    return self->mode();
+    LOCK(auto ret = self->mode());
+    return ret;
 }
 
 void Fl_Glut_Window_set_mode(Fl_Glut_Window *self, int mode) {
-    self->mode(mode);
+    LOCK(self->mode(mode);)
 }
 
 void *Fl_Glut_Window_get_proc_address(Fl_Glut_Window *self, const char *s) {
-    return (void *)glutGetProcAddress(s);
+    LOCK(auto ret = (void *)glutGetProcAddress(s)); return ret;
 }
 
 #endif
